<?php
// 'product' object
class Product{

	// database connection and table name
	private $conn;
	private $table_name="products";

	// object properties
	public $id;
	public $name;
	public $price;
	public $stock;
	public $description;
	public $category_id;
	public $category_name;
	public $timestamp;

	// constructor
	public function __construct($db){
		$this->conn = $db;
	}

	// update stock
	function updateStock(){

		// product update query
		$query = "UPDATE
					" . $this->table_name . "
				SET
					stock = :stock
				WHERE
					id = :id";

		// prepare query statement
		$stmt = $this->conn->prepare($query);

		// sanitize
		$this->stock=htmlspecialchars(strip_tags($this->stock));
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind variable values
		$stmt->bindParam(':stock', $this->stock);
		$stmt->bindParam(':id', $this->id);

		// execute the query
		if($stmt->execute()){
			return true;
		}else{
			return false;
		}
	}

	// used when filling up the update product form
	function readOne(){

		// query to select single record
		$query = "SELECT
					c.name as category_name, p.name, p.description, p.price, p.stock, p.category_id
				FROM
					" . $this->table_name . " p
					LEFT JOIN
						categories c
							ON p.category_id = c.id
				WHERE
					p.id = ?
				LIMIT
					0,1";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// sanitize
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind product id value
		$stmt->bindParam(1, $this->id);

		// execute query
		$stmt->execute();

		// get row values
		$row = $stmt->fetch(PDO::FETCH_ASSOC);

		// assign retrieved row value to object properties
		$this->name = $row['name'];
		$this->description = $row['description'];
		$this->price = $row['price'];
		$this->stock = $row['stock'];
		$this->category_id = $row['category_id'];
		$this->category_name = $row['category_name'];
	}

	// read all product based on product ids included in the $ids variable
	// reference http://stackoverflow.com/a/10722827/827418
	public function readByIds($ids){

		$ids_arr = str_repeat('?,', count($ids) - 1) . '?';

		// query to select products
		$query = "SELECT id, name, price, stock FROM products WHERE id IN ({$ids_arr}) ORDER BY name";

		// prepare query statement
		$stmt = $this->conn->prepare($query);

		// execute query
		$stmt->execute($ids);

		// return values from database
		return $stmt;
	}

	// read all products
	function search($search_term, $from_record_num, $records_per_page){

		// select all products query
		$query = "SELECT
					c.name as category_name, p.id, p.name, p.description, p.price, p.stock, p.category_id
				FROM
					" . $this->table_name . " p
					LEFT JOIN
						categories c
					ON
						p.category_id = c.id
				WHERE
					p.name LIKE :name
					OR p.description LIKE :description
				ORDER BY
					p.created DESC
				LIMIT
					:from_record_num, :records_per_page";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// sanitize
		$search_term=htmlspecialchars(strip_tags($search_term));

		// search term
		$search_term = "%{$search_term}%";

		// bind search term
		$stmt->bindParam(":name", $search_term);
		$stmt->bindParam(":description", $search_term);
		$stmt->bindParam(":from_record_num", $from_record_num, PDO::PARAM_INT);
		$stmt->bindParam(":records_per_page", $records_per_page, PDO::PARAM_INT);

		// execute query
		$stmt->execute();

		// return values
		return $stmt;
	}

	// used for paging products
	public function countBySearch($search_term){

		// query to count all product records
		$query = "SELECT count(*) FROM " . $this->table_name . " WHERE name=:name OR description=:description";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// sanitize
		$search_term=htmlspecialchars(strip_tags($search_term));

		// search term
		$search_term = "%{$search_term}%";

		// bind search term
		$stmt->bindParam(":name", $search_term);
		$stmt->bindParam(":description", $search_term);

		// execute query
		$stmt->execute();

		// get row value
		$rows = $stmt->fetch(PDO::FETCH_NUM);

		// return count
		return $rows[0];
	}

	// read by category id
	function readByCategoryId($from_record_num, $records_per_page){

		// select all products query
		$query = "SELECT
					c.name as category_name, p.id, p.name, p.description, p.price, p.stock, p.category_id
				FROM
					" . $this->table_name . " p
					LEFT JOIN
						categories c
					ON
						p.category_id = c.id
				WHERE
					c.id = ?
				ORDER BY
					p.created DESC
				LIMIT
					?, ?";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// bind limit clause variables
		$stmt->bindParam(1, $this->category_id, PDO::PARAM_INT);
		$stmt->bindParam(2, $from_record_num, PDO::PARAM_INT);
		$stmt->bindParam(3, $records_per_page, PDO::PARAM_INT);

		// execute query
		$stmt->execute();

		// return values
		return $stmt;
	}

	// read all products
	function read($from_record_num, $records_per_page){

		// select all products query
		$query = "SELECT
					c.name as category_name, p.id, p.name, p.description, p.price, p.stock, p.category_id
				FROM
					" . $this->table_name . " p
					LEFT JOIN
						categories c
					ON
						p.category_id = c.id
				ORDER BY
					p.created DESC
				LIMIT
					?, ?";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// bind limit clause variables
		$stmt->bindParam(1, $from_record_num, PDO::PARAM_INT);
		$stmt->bindParam(2, $records_per_page, PDO::PARAM_INT);

		// execute query
		$stmt->execute();

		// return values
		return $stmt;
	}

    // used for paging products
	public function count(){

		// query to count all product records
		$query = "SELECT count(*) FROM " . $this->table_name;

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// execute query
		$stmt->execute();

		// get row value
		$rows = $stmt->fetch(PDO::FETCH_NUM);

		// return count
		return $rows[0];
	}

	// used for paging products retrieved by category id
	public function countByCategoryId(){

		// query to count all product records
		$query = "SELECT count(*) FROM " . $this->table_name . " WHERE category_id = ?";

		// prepare query statement
		$stmt = $this->conn->prepare( $query );

		// sanitize
		$this->category_id=htmlspecialchars(strip_tags($this->category_id));

		// bind selected record id
		$stmt->bindParam(1, $this->category_id);

		// execute query
		$stmt->execute();

		// get row value
		$rows = $stmt->fetch(PDO::FETCH_NUM);

		// return count
		return $rows[0];
	}
}
?>
